"""Ratio theme for Typst

Generate Typst documents from Python using the Ratio theme.
"""

from ratio_typst.document import Author, Content, Info, Kind, Report, Slides, String, Theme

__all__ = [Author, Content, Info, Kind, Report, Slides, String, Theme]

__version__ = "0.1.3"
